/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  */
// Includes ------------------------------------------------------------------
#include "main.h"
#include "stm32f1xx_hal.h"

/* USER CODE BEGIN Includes */
#include <string.h>
#include <math.h>
/* USER CODE END Includes */

// Private variables ---------------------------------------------------------
SPI_HandleTypeDef hspi1;
SPI_HandleTypeDef hspi2;
DMA_HandleTypeDef hdma_spi1_tx;
DMA_HandleTypeDef hdma_spi1_rx;
DMA_HandleTypeDef hdma_spi2_tx;

/* USER CODE BEGIN PV */
// Private variables ---------------------------------------------------------
// day, month, year, audio channels, amplifier present
FwVersion     fw_version = { 2, 2, 2018, 2, 0 };

volatile u32  blink_timeout, activity_timeout, spi_timeout;
struct SavedOptions LocalOptions;

union
{
  Surface       sf;
  Surface_bits  sfb;
} spi_tx_data;
SrfCommand    spi_cmd;

bool          is_volume_mode = true;
bool          read_surface = false;
u8            volume_points[256];
Volume        max_volume, current_volume;
/* USER CODE END PV */

// Private function prototypes -----------------------------------------------
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_DMA_Init(void);
static void MX_SPI1_Init(void);
static void MX_SPI2_Init(void);

/* USER CODE BEGIN PFP */
// Private function prototypes -----------------------------------------------
void          NV_Options( void );
int           Program_Options( int id );
/* USER CODE END PFP */

/* USER CODE BEGIN 0 */
bool
init_log_scale( int points )
{
  int           i, j, v = 0;
  float         top = log( 255.9 );
  if( points < 5 || points > 100 )
    return false;
  for( i = 0; i <= points; i++ )
    volume_points[i] = exp( top * i / points ) - .5;
  // compact same integer values
  for( j = 0, i = 1; i <= points; i++ )
  {
    while( volume_points[i + j] == v )
      j++;
    if( j )
    {
      memmove( &volume_points[i], &volume_points[i + j], points + 1 - i - j );
      points -= j;
      j = 0;
    }
    v = volume_points[i];
  }
  LocalOptions.volume_steps = points;
  if( LocalOptions.volume_last > LocalOptions.volume_steps )
    LocalOptions.volume_last = LocalOptions.volume_steps;
  return true;
}

struct __attribute__ ( ( packed, aligned( 4 ) ) )
{
  u8            v1, c1, v2, c2;
} mcp42xxx;

void
HAL_SPI_TxCpltCallback( SPI_HandleTypeDef * hspi )
{
  pinSet( POT_CS_GPIO_Port, POT_CS_Pin );	// CS inactive
  if( mcp42xxx.c1 == 0x11 )	// end update
    return;
  mcp42xxx.c1 = 0x11;
  mcp42xxx.v1 = current_volume.R2;	// right channel
  mcp42xxx.c2 = 0x11;
  mcp42xxx.v2 = current_volume.R1;
  pinReset( POT_CS_GPIO_Port, POT_CS_Pin );	// CS active
  HAL_SPI_Transmit_DMA( &hspi2, ( u8 * ) & mcp42xxx, 2 );
}

void
update_volume( void )
{
  int           i;
  u8           *s, *d;
  u16           enc_vol = 1 + volume_points[LocalOptions.volume_last];
  spi_timeout = 20;
  for( s = ( u8 * ) & max_volume, d = ( u8 * ) & current_volume,
      i = 0; i < 4; i++, s++, d++ )
    *d = ( enc_vol * *s ) / 256;
  // NSS must be GPIO to prevent it to toggle between frames
  mcp42xxx.c1 = 0x12;
  mcp42xxx.v1 = current_volume.L2;	// left channel
  mcp42xxx.c2 = 0x12;
  mcp42xxx.v2 = current_volume.L1;
  pinReset( POT_CS_GPIO_Port, POT_CS_Pin );	// CS active
  HAL_SPI_Transmit_DMA( &hspi2, ( u8 * ) & mcp42xxx, 2 );
}

#define SWITCH_MASK  ( (1U<<20) - 1 )	// 20 ms debounce for switches

u32           sw_state[5+2+5];
u32           enc_state[2*5];

inline static void
shift_state( void )
{
  int i = 0;
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( SwE1_GPIO_Port, SwE1_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( SwE2_GPIO_Port, SwE2_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( SwE3_GPIO_Port, SwE3_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( SwE4_GPIO_Port, SwE4_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( SwE5_GPIO_Port, SwE5_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( Sw1_GPIO_Port, Sw1_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( Sw2_GPIO_Port, Sw2_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( Sw3_GPIO_Port, Sw3_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( Sw4_GPIO_Port, Sw4_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( Sw5_GPIO_Port, Sw5_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( Sw6_GPIO_Port, Sw6_Pin );
  sw_state[i] <<= 1;
  sw_state[i++] |= pinRead( Sw7_GPIO_Port, Sw7_Pin );
  i = 0;
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc1A_GPIO_Port, Enc1A_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc1B_GPIO_Port, Enc1B_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc2A_GPIO_Port, Enc2A_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc2B_GPIO_Port, Enc2B_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc3A_GPIO_Port, Enc3A_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc3B_GPIO_Port, Enc3B_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc4A_GPIO_Port, Enc4A_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc4B_GPIO_Port, Enc4B_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc5A_GPIO_Port, Enc5A_Pin );
  enc_state[i] <<= 1;
  enc_state[i++] |= pinRead( Enc5B_GPIO_Port, Enc5B_Pin );
}

inline static void
signal_data_ready( void )
{
    pinReset( Srf_Int_GPIO_Port, Srf_Int_Pin );
    activity_timeout = 20;
    pinSet( LED_GPIO_Port, LED_Pin );
    pinSet( Srf_Int_GPIO_Port, Srf_Int_Pin );
}

#define ENC_PULSE 8
#define ENC_LOW_MASK ((1<<ENC_PULSE)-1)
#define ENC_HIGH_MASK (ENC_LOW_MASK << ENC_PULSE )

void
switches( void )
{
  bool          changed = false;
  u32           sw = spi_tx_data.sfb.switches;
  u32           enc = spi_tx_data.sfb.encoders;
  int           i, steps, dir;

  shift_state();
  for( i = 0; i < 12; i++ )
  {
    if( ( sw_state[i] & SWITCH_MASK ) == 0 )	// switch pressed
    {
      if( !( sw & ( 1 << i ) ) )	// not yet marked pressed
      {
	sw |= ( 1 << i );
	changed = true;
      }
      continue;
    }
    if( ( sw_state[i] & SWITCH_MASK ) == SWITCH_MASK )	// switch depressed
    {
      if( sw & ( 1 << i ) )	// not yet marked depressed
      {
	sw &= ~( 1 << i );
	changed = true;
      }
      continue;
    }
  }
  for( i = 0; i < 5; i++ )
  {
    if( enc_state[i<<1] == -1 || enc_state[i<<1] == 0 )
      continue;
    if( ( enc_state[i<<1] & ENC_HIGH_MASK ) == ENC_HIGH_MASK &&
      ( ( enc_state[i<<1] & ENC_LOW_MASK ) == 0 ) )
    {
      // dir = enc_state[i<<1|1] & ( 1 << __builtin_ctz( enc_state[i<<1] ) );
      // enc_state[i<<1] = 0;
      dir = enc_state[i<<1|1] & (1<<ENC_PULSE);
      if( i == 4 && is_volume_mode == true )
      {
	if( dir )
	{
	  if( LocalOptions.volume_last == LocalOptions.volume_steps )
	    continue;
	  LocalOptions.volume_last++;;
	}
	else
	{
	  if( LocalOptions.volume_last == 0 )
	    continue;
	  LocalOptions.volume_last--;
	}
	update_volume(  );
	continue;
      }
      steps = ( enc >> (i<<2) ) & 0xf;
      if( steps & 8 ) steps |= ~0xf; // extend sign
      if( dir )
      {
	if( steps == 7 )
	  continue;
	steps++;
        changed = true;
      }
      else
      {
	if( steps == -8 )
	  continue;
	steps--;
        changed = true;
      }
      if( changed )
      {
        enc = ( enc & ~( 0xf << (i<<2) ) ) | ( steps & 0xf ) << (i<<2);
      }
    }
  }
  spi_tx_data.sfb.switches = sw;
  spi_tx_data.sfb.encoders = enc;
  if( changed )
  {
    // re-init the DMA transfer
    while( HAL_SPI_TransmitReceive_DMA( &hspi1, ( u8 * ) & spi_tx_data,
	( u8 * ) & spi_cmd, 2 ) != HAL_OK )
      HAL_SPI_Abort( &hspi1 );
    signal_data_ready();
  }
}

void
HAL_SYSTICK_Callback( void )
{
  int           ms;

  if( read_surface == false )
    return;
  switches(  );			// read all switches every milisecond
  if( spi_timeout )
  {
    spi_timeout--;
    if( spi_timeout == 0 )
    {
      if( is_volume_mode )
	pinSet( VolumeMode_GPIO_Port, VolumeMode_Pin );
      else
	pinReset( VolumeMode_GPIO_Port, VolumeMode_Pin );
    }
  }
  if( activity_timeout )
  {
    activity_timeout--;
    if( activity_timeout == 0 )
      pinReset( LED_GPIO_Port, LED_Pin );
  }
  if( blink_timeout )		// for 'blink_timeout' ms flash the LED with 2Hz
  {
    ms = -blink_timeout % 500;
    if( ms == 0 )
      pinSet( LED_GPIO_Port, LED_Pin );
    else if( ms == 20 )
      pinReset( LED_GPIO_Port, LED_Pin );
    blink_timeout--;
  }
}

void
SPI_activity( void )
{
  // show SPI activity on the Volume LED
  if( is_volume_mode )
    pinReset( VolumeMode_GPIO_Port, VolumeMode_Pin );
  else
    pinSet( VolumeMode_GPIO_Port, VolumeMode_Pin );
  spi_timeout = 20;
}

void
HAL_SPI_TxRxHalfCpltCallback( SPI_HandleTypeDef * hspi )
{
  SPI_activity(  );
}

void
HAL_SPI_RxHalfCpltCallback( SPI_HandleTypeDef * hspi )
{
  SPI_activity(  );
}

void
End_Of_Transfer( SPI_HandleTypeDef * hspi )
{
  Surface_bits    *sfb = &spi_tx_data.sfb;
  pinReset( Srf_Int_GPIO_Port, Srf_Int_Pin );	// clear the interrupt
  switch ( spi_cmd.command )
  {
    case GET_FW_VERSION:
      // redirect to firmware version for the next transaction
      while( HAL_SPI_TransmitReceive_DMA( hspi, ( u8 * ) & fw_version,
	  ( u8 * ) & spi_cmd, 2 ) != HAL_OK )
	HAL_SPI_Abort( hspi );
      signal_data_ready();
      return;
    // commands bellow this point don't return any data in a following transfer
    case READ_SRF_CLEAR:
      while( HAL_SPI_TransmitReceive_DMA( hspi, ( u8 * ) & spi_tx_data,
	  ( u8 * ) & spi_cmd, 2 ) != HAL_OK )
	HAL_SPI_Abort( hspi );
      sfb->encoders = 0;
      return;
    case ENC5_DATA:
      is_volume_mode = false;
      pinReset( VolumeMode_GPIO_Port, VolumeMode_Pin );
      break;
    case ENC5_VOLUME:
      is_volume_mode = true;
      pinSet( VolumeMode_GPIO_Port, VolumeMode_Pin );
      break;
    case SET_MAX_VOLUMES_CH1:
      max_volume.L1 = ( u8 ) spi_cmd.param;
      max_volume.R1 = ( u8 ) ( spi_cmd.param >> 8 );
      update_volume(  );
      break;
    case SET_MAX_VOLUMES_CH2:
      max_volume.L2 = ( u8 ) spi_cmd.param;
      max_volume.R2 = ( u8 ) ( spi_cmd.param >> 8 );
      update_volume(  );
      break;
    case SET_VOLUME_STEPS:
      if( init_log_scale( spi_cmd.param ) )
      {
	LocalOptions.id++;
	Program_Options( ++LocalOptions.id );
	update_volume(  );
      }
      break;
  }
  // enable new transfer
  while( HAL_SPI_Receive_DMA( hspi, ( u8 * ) & spi_cmd, 2 ) != HAL_OK )
    HAL_SPI_Abort( hspi );
}

void
HAL_SPI_ErrorCallback(SPI_HandleTypeDef * hspi)
{
  HAL_SPI_Abort( hspi );
  // enable new transfer
  while( HAL_SPI_Receive_DMA( hspi, ( u8 * ) & spi_cmd, 2 ) != HAL_OK )
    HAL_SPI_Abort( hspi );
}

void
HAL_SPI_TxRxCpltCallback( SPI_HandleTypeDef * hspi )
{
  End_Of_Transfer( hspi );
}

void
HAL_SPI_RxCpltCallback( SPI_HandleTypeDef * hspi )
{
  End_Of_Transfer( hspi );
}
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  *
  * @retval None
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  NV_Options(  );

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_SPI1_Init();
  MX_SPI2_Init();
  /* USER CODE BEGIN 2 */
  // wait for SPI transfer from STM32F4 (32bits)
  pinSet( Srf_Int_GPIO_Port, Srf_Int_Pin );	// mark ready
  while( HAL_SPI_Receive_DMA( &hspi1, ( u8 * ) & spi_cmd, 2 ) != HAL_OK )
    HAL_SPI_Abort( &hspi1 );
  read_surface = true;
  blink_timeout = 3000;
  is_volume_mode = true;
  pinSet( VolumeMode_GPIO_Port, VolumeMode_Pin );
  max_volume.L1 = max_volume.R1 = max_volume.L2 = max_volume.R2 = 255;	// max volume per channel
  init_log_scale( LocalOptions.volume_steps );
  update_volume(  );

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while( 1 )
  {
  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */
    // sleep all the time - code is interrupt driven
    HAL_PWR_EnterSLEEPMode( PWR_MAINREGULATOR_ON, PWR_SLEEPENTRY_WFI );
  }
  /* USER CODE END 3 */

}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;

    /**Initializes the CPU, AHB and APB busses clocks
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 16;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Initializes the CPU, AHB and APB busses clocks
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Configure the Systick interrupt time
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 2, 0);
}

// SPI1 init function
static void MX_SPI1_Init(void)
{

  /* SPI1 parameter configuration*/
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_SLAVE;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_16BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_HARD_INPUT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_4;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}

// SPI2 init function
static void MX_SPI2_Init(void)
{

  /* SPI2 parameter configuration*/
  hspi2.Instance = SPI2;
  hspi2.Init.Mode = SPI_MODE_MASTER;
  hspi2.Init.Direction = SPI_DIRECTION_1LINE;
  hspi2.Init.DataSize = SPI_DATASIZE_16BIT;
  hspi2.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi2.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi2.Init.NSS = SPI_NSS_SOFT;
  hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_4;
  hspi2.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi2.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi2.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi2.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi2) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}

/**
  * Enable DMA controller clock
  */
static void MX_DMA_Init(void)
{
  // DMA controller clock enable
  __HAL_RCC_DMA1_CLK_ENABLE();

  /* DMA interrupt init */
  /* DMA1_Channel2_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel2_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(DMA1_Channel2_IRQn);
  /* DMA1_Channel3_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel3_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(DMA1_Channel3_IRQn);
  /* DMA1_Channel5_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(DMA1_Channel5_IRQn, 1, 1);
  HAL_NVIC_EnableIRQ(DMA1_Channel5_IRQn);

}

/** Configure pins as
        * Analog
        * Input
        * Output
        * EVENT_OUT
        * EXTI
        * Free pins are configured automatically as Analog (this feature is enabled through
        * the Code Generation settings)
*/
static void MX_GPIO_Init(void)
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(POT_CS_GPIO_Port, POT_CS_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(VolumeMode_GPIO_Port, VolumeMode_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, Srf_Int_Pin|LED_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pins : Sw4_Pin Sw6_Pin Sw3_Pin */
  GPIO_InitStruct.Pin = Sw4_Pin|Sw6_Pin|Sw3_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pins : PD0 PD1 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);

  /*Configure GPIO pins : Enc1B_Pin Enc2B_Pin Enc3B_Pin Enc4B_Pin
                           Enc5B_Pin Sw5_Pin Sw7_Pin */
  GPIO_InitStruct.Pin = Enc1B_Pin|Enc2B_Pin|Enc3B_Pin|Enc4B_Pin
                          |Enc5B_Pin|Sw5_Pin|Sw7_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : Enc1A_Pin Enc2A_Pin Enc3A_Pin Sw1_Pin
                           Sw2_Pin Enc4A_Pin Enc5A_Pin SwE1_Pin
                           SwE2_Pin SwE3_Pin SwE4_Pin SwE5_Pin */
  GPIO_InitStruct.Pin = Enc1A_Pin|Enc2A_Pin|Enc3A_Pin|Sw1_Pin
                          |Sw2_Pin|Enc4A_Pin|Enc5A_Pin|SwE1_Pin
                          |SwE2_Pin|SwE3_Pin|SwE4_Pin|SwE5_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pin : POT_CS_Pin */
  GPIO_InitStruct.Pin = POT_CS_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_MEDIUM;
  HAL_GPIO_Init(POT_CS_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : VolumeMode_Pin */
  GPIO_InitStruct.Pin = VolumeMode_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(VolumeMode_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : PA8 */
  GPIO_InitStruct.Pin = GPIO_PIN_8;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : Srf_Int_Pin */
  GPIO_InitStruct.Pin = Srf_Int_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(Srf_Int_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : LED_Pin */
  GPIO_InitStruct.Pin = LED_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(LED_GPIO_Port, &GPIO_InitStruct);

  /*Configure peripheral I/O remapping */
  __HAL_AFIO_REMAP_PD01_ENABLE();

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  file: The file name as string.
  * @param  line: The line in file as a number.
  * @retval None
  */
void _Error_Handler(char *file, int line)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while( 1 )
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number, printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

